<?php
/*
 * Plugin Name: SEO Facebook Comments
 * Plugin URI: http://www.pazzanitech.com.br
 * Description: This plugin gets the comments of your page from facebook shows it on the Post and also Inputs it in the wordpress database (comments table)
 * Version: 1.2
 * Author: Fabio Zaffani  
 * Author URI: http://www.pazzanitech.com.br
 * License: GPL2 
 * 
 * Copyright 2011  Fabio Alves Zaffani ( email : fabiozaffani@gmail.com ) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

// Make sure there is no bizarre coincidence of someone creating a class with the exactly same name of this plugin
if ( !class_exists("SEOFacebookComments") )
{
    // Insert Facebook PHP Class, but first make sure no other plugins had done that before
    if ( !class_exists('Facebook') )
	    require_once ( WP_PLUGIN_DIR . '/seo-facebook-comments/facebook/facebook.php' );
	
	class SEOFacebookComments
	{
		protected $name;                // Plugin name
		protected $pluginUrl;           // Plugin location
        protected $pluginAdminPage;     // Plugin Admin Page Config
		protected $fb;                  // Plugin Options from options table
        protected $table;               // Plugin db table name

		/**
		 * Lets set some default values to used in the plugin and also appends all that is
		 * necessary to the Wordpress Hooks
		 * @return void
		 */
		public function __construct()
		{
			$this->name = 'fbseocomments';
			$this->fb = get_option('fbseocomments');
			$this->pluginUrl = WP_PLUGIN_DIR . '/seo-facebook-comments/';
            $this->pluginAdminPage = 'options-general.php?page=fbseocomments';
            $this->table = 'wp_comments_fbseo'; # @TODO this should not be wp_, that info should be get from wp configuration

			// @ref http://codex.wordpress.org/Function_Reference/add_action
			add_action( 'wp_print_styles', array( &$this, 'addFbStyles' ));
			add_action( 'admin_menu', array( &$this, 'page' ) );
			
			// @ref http://codex.wordpress.org/Function_Reference/register_activation_hook
			register_activation_hook( __FILE__, array( &$this, 'install' ));
			
			// @ref http://codex.wordpress.org/Function_Reference/register_deactivation_hook
			register_deactivation_hook( __FILE__, array( &$this, 'remove' ));

            // @TODO create option so user can choose to embed or not facebook language and open graph
			// og tags
			add_action(	'wp_head',	array( &$this, 'fbOpenGraph' ) ); // og tag
			
			// fbLanguages
			add_action(	'language_attributes', array( &$this, 'fbLanguages' ));

			// comments
			add_action( 'admin_init', array (&$this, 'register' ));
			add_action(	'wp_head', array( &$this, 'loadJS' ));
			add_filter(	'comments_array',	array( &$this, 'FbComments' ));

            // Show a msg to the user telling that he needs to update the Plugin Options
            if ( empty($this->fb['app']) || empty($this->fb['secret']) || empty($this->fb['admin']) )
                add_action('admin_notices', array( &$this, 'welcomeMessage') );

            // If the user allowed the plugin to be shared, will embed a link in the footer
            if ( $this->fb['share'] )
                add_action( 'wp_footer', array( &$this, 'share' ));
		}

        /**
         *
         * This method insert the default config of the plugin and also creates a database
         * to control what comments were already added and those that were not
         * @return void
         */
        public function install()
        {
            global $wpdb;

            $defaults = array(
                    'app'						=> '',
                    'secret'					=> '',
                    'admin'						=> '',
                    'language'					=> 'en_US',
                    'numPosts'					=> 10,
                    'width'						=> '500px',
                    'share'                     => 1,
                    'colorScheme'				=> 'light',
                    'autoApprove'				=> 0,
                    'hideComments'				=> 0,
                    'commentsWrapper'            => 'comments',
                    'reverseOrder'				=> 0,
                    'dashnumComments'			=> 10,
                    'fbCommentCount'			=> 1
            );

            update_option( $this->name, $defaults );

            // Create a new DB to keep track of the new vs already added facebook comments
            $fbSql = "CREATE TABLE IF NOT EXISTS `{$this->table}` (
                        `id` bigint(20) unsigned NOT NULL auto_increment,
                        `comment_ID` bigint(20) unsigned NOT NULL,
                        `facebook_comment_ID` bigint(20) unsigned NOT NULL,
                        `facebook_user_ID` bigint(20) unsigned NOT NULL,
                        PRIMARY KEY (`id`)
                        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

            $wpdb->query($fbSql);

        }

		/**
		 *
		 * This method could remove the database used by this plugin when it is removed, not advisable however
		 * @TODO description on what to do is inside
		 * @return void
		 */
		public function remove()
		{
            // Lets remove the plugin options from the wp_options table, pluigns should keep things clean!
            // ref http://codex.wordpress.org/Function_Reference/delete_option
            delete_option( $this->name );

            /*
             *
             * ask the user if the database should be removed
             * If yes
             *  should also remove all the created comments (so avoid duplication in a posterior plugin installation)
             *
             * If no
             *  alert the user about the issues that might occur by not deleting this
             *
             * FUTURE make the plugin inteligente enought to not remove the comments and if reinstalled will only add the
             * new found comments (may have some performance issues)
             *
             */
		}

        /**
         * Method to show a welcome message to the user after he installs the plugin
         * @return void
         */
        public function welcomeMessage()
        {
            // Check if user has the right privileges to see that warning message
            if ( !current_user_can('administrator') )
                return;

            echo '<div class="updated"><p><b>SEO Facebook Comments is almost ready</b> to use, however you MUST insert the<a href="' . $this->pluginAdminPage . '"> <strong>APP ID, App Secret and Admin E-mail</strong> </a></p></div>';
        }

		/**
		 *
		 * This method sends the styles used for the comments to the page
		 * @return void
		 */
		public function addFbStyles()
		{
			$fbSEOStyleURL = plugins_url( 'fbSEOStyle.css' , __FILE__ ); # Respects SSL, Style.css is relative to the file
			$seoStyle = $this->pluginUrl . 'fbSEOStyle.css';

			if ( file_exists($seoStyle) )
			{
				// http://codex.wordpress.org/Function_Reference/wp_register_style
				wp_register_style('fbSEOStylesheet', $fbSEOStyleURL);
				// @ref http://codex.wordpress.org/Function_Reference/wp_enqueue_style
				wp_enqueue_style( 'fbSEOStylesheet');
			}

            // Lets check if the user wants to hide the default comments
            if ( $this->fb['hideComments'] )
            {
                $wpCommentURL = plugins_url( 'wpcomments.css' , __FILE__ ); // Respects SSL, Style.css is relative to the file
			    $wpCommentStyle = $this->pluginUrl . 'wpcomments.css';

                if( !file_exists($wpCommentStyle) )
                       return;

				wp_register_style('fbSEOwpcomments', $wpCommentURL);
				wp_enqueue_style( 'fbSEOwpcomments');
            }
		}

        /**
		 *
		 * Method that register the Plugin configs on the wp_options table
		 * this method must be called on the admin_init hook otherwise this class will trigger an
		 * Error: options page not found on save
		 * @return void
		 */
		public function register()
		{
			register_setting( 'facebook_group', $this->name );
		}

		/**
		 * 
		 * This method creates links on the footer pointing to where the pugin were created
         * @return void
		 */
		public function share()
		{
			$output = '<div id="sharefbseo">';
			$output .= 'Plugin from the creators of <a href="http://www.pazzanibrindes.com.br" target="_blank">Brindes Personalizados</a> :: More at <a href="http://www.pazzanitech.com.br" target="_blank">Pazzani Technologies</a>';
			$output .= '</div>';
			echo $output;
		}
		
		/**
		 * 
		 * Method to add the facebook in the Settings section of the admin menu panel
		 * @return void
		 */
		public function page()
		{
			add_options_page( 'SEO Facebook Comments Configuration', 'SEO Facebook Comments', 'administrator', $this->name, array( &$this, 'adminOutputHTML'));
		}

		/**
		 * 
		 * This methods creates checkboxes for the SEO Facebook Comments Admin Config Page
		 * @param array $options
		 * @return false on error
         * @return void
		 */
		public function adminPrintCheck($options)
		{
			if( !is_array($options) || empty($options) )
				return false;
				
			$name = $options[0];
			$obs = $options[1];
			
			if ( $this->fb[$name] == '1')
				$checked = 'checked="checked" ';
			else
				$checked = '';
				
			$html = "<input type=\"checkbox\" name=\"{$this->name}[{$name}]\" value=\"1\" {$checked}/>";

			if(isset($obs) && !empty($obs))
				$html .= "<br/><small>{$obs}</small>";

			echo $html;

		}
		
		/**
		 * 
		 * This methods prints a select box on the SEO Facebook Comments Admin Config Page
		 * @param array $options
		 * @return false on error
         * @return void
		 */
		public function adminPrintSelect($options)
		{
			if( !is_array($options) || empty($options) )
				return false;
			
			$html = "<select name=\"{$this->name}[colorScheme]\">";
			foreach ($options as $option)
			{
				if ( $this->fb['colorScheme'] == $option)
					$selected = 'selected="selected" ';
				else
					$selected = '';
				$html .= "<option value=\"{$option}\" {$selected}>{$option}</option>";
			}
			
			$html .= '</select>';
			echo $html;
			
		}
		
		/**
		 * 
		 * Method to output title fields on the SEO Facebook Comments Admin Config Page
		 * @return void
		 */
		public function adminPrintTitle()
		{
			echo "<small>Need Help? Visit <a href=\"http://www.pazzanitech.com.br/how-to-create-a-facebook-app\" target=\"_blank\">How to Create a Facebook APP</a> at our blog</small>";
		}
		
		/**
		 * 
		 * Method to output the input fields for the SEO Facebook Comments Admin Page
		 * @param string/array $fields
		 * @return void
		 */
		public function aminPrintInput($fields)
		{
			if(is_array($fields))
			{
				$name = $fields[0];
				$obs = $fields[1];
			}
			else
				$name = $fields;

			$options = get_option($this->name);
			$html = "<input type=\"text\" name=\"{$this->name}[{$name}]\" size=\"60\" value=\"{$options[$name]}\" />";

			if (!empty($obs))
				$html .= '<br/><small>' . $obs . '</small>';

			echo $html;
		}
		
		/**
		 * 
		 * Method to register the options available in the configs page of the Facebook Comments SEO
		 * @return void
		 */
		public function adminOutputHTML()
		{
			$sectionGeral = 'fbSEOCommentsGeneral';
			$sectionComments = 'FbComments';
			$titleCall = array( &$this, 'adminPrintTitle' );
			$inputCall = array( &$this,  'aminPrintInput' );
			$selectCall = array( &$this, 'adminPrintSelect' );
			$checkCall = array( &$this, 'adminPrintCheck');
			
			// Admin Page General Configuration
			add_settings_section($sectionGeral, 'Plugin General Config', $titleCall , $this->name);
				add_settings_field( 'app', '<strong>APP ID</strong>', $inputCall, $this->name, $sectionGeral, 'app');
				add_settings_field( 'secret', '<strong>APP Secret</strong>', $inputCall, $this->name, $sectionGeral, 'secret');
				add_settings_field( 'admin', '<strong>Admin Email</strong>', $inputCall, $this->name, $sectionGeral, 'admin');
				add_settings_field( 'language', 'Language', $inputCall, $this->name, $sectionGeral, 'language');
                add_settings_field( 'share', 'Share the Love', $checkCall, $this->name, $sectionGeral, array('share', 'Help us make great plugins like this one by sharing our link'));

			// Admin Page Coments Configuration
			add_settings_section($sectionComments, 'Plugin Comments Config', $titleCall , $this->name);
				add_settings_field( 'numPosts', 'Posts Number', $inputCall, $this->name, $sectionComments,'numPosts');
				add_settings_field( 'width', 'Comments Width', $inputCall, $this->name, $sectionComments, array( 'width', 'The width can be in px. Example: 500px'));
				add_settings_field( 'colorScheme', 'Color Scheme', $selectCall, $this->name, $sectionComments, array( 'light', 'dark') );
				add_settings_field( 'autoApprove', 'Auto Approve', $checkCall, $this->name, $sectionComments, array('autoApprove', 'Check this box if you want your Facebook Comments to be auto-approved.' ));
				add_settings_field( 'hideComments', 'Hide Default Comments', $checkCall, $this->name, $sectionComments, array('hideComments', 'Check this box if you want your Wordpress comments to be hidden (only the Crawlers will see it).' ));

			echo "<form method='post' action='options.php'>";
			settings_fields('facebook_group');
			do_settings_sections($this->name);
			echo "<p class='submit'>";
			echo "<input type='submit' class='button-primary' value='";
			echo  _e('Save Changes');
			echo "' /></p></form>";
		}

		/**
		 *
		 * Insert the tags in the <html> on the head, it also helps with IE compatibility
		 * @return string $attributes
		 */
		public function fbLanguages( $attributes='' )
		{
				$attributes .= ' xmlns:fb="http://www.facebook.com/2008/fbml"';
				$attributes .= ' xmlns:og="http://opengraphprotocol.org/schema/"';
				return $attributes;
		}

		/**
		 *
		 * Add the open graph tags to the header of the page, creating an Facebook Object
		 * @return void
		 */
		public function fbOpenGraph()
		{
			global $wp_query;

			$postId = $wp_query->post->ID;
			$postUrl = get_permalink($postId);
			$siteName = get_bloginfo('name');
			$appId = $this->fb['app'];

			if ( is_home() || is_search() )
            {
				$postTitle = $siteName;
            }
            else if ( is_category() )
			{
				$category = get_the_category();
				$postTitle = $category[0]->cat_name . ' - ' . $siteName;
			}
			else
				$postTitle = single_post_title('', false);

			echo "<meta property='fb:admins' content='{$this->fb['admin']}' />";
			echo "<meta property='og:title' content='{$postTitle}' />",
				 "<meta property='og:site_name' content='{$siteName}' />",
				 "<meta property='og:url' content='{$postUrl}' />",
				 "<meta property='og:type' content='article' />",
				 "<meta property='fb:app_id' content='{$appId}'>\n";
		}

		/**
		 * This method uses the facebook php togheter with the APP info provided by the user to instantiate a new
		 * Facebook object
		 * @return object Facebook;
		 */
		public function getFbCommentsApi()
		{
			$fbApiCredentials = array(
				'appId'	 => $this->fb['app'],
				'secret' => $this->fb['secret'],
				'FILEUPLOAD' => false
			);

			return new Facebook($fbApiCredentials);
		}

		public function loadJS()
		{
			$appID = $this->fb['app'];
			$lang = $this->fb['language'];

			$js = "	<div id=\"fb-root\"></div>
				<script>
				  window.fbAsyncInit = function() {
					FB.init({
					  appId  : {$appID},
					  status : true, // check login status
					  cookie : true, // enable cookies to allow the server to access the session
					  xfbml  : true  // parse XFBML
					});
				  };

				  (function() {
					var e = document.createElement('script');
					e.src = document.location.protocol + '//connect.facebook.net/{$lang}/all.js';
					e.async = true;
					document.getElementById('fb-root').appendChild(e);
				  }());
				</script>";

               echo $js;
		}

		/**
		 * 
		 * This method filters the default comments of wordpress adding the facebook originated content
		 * @param array $comments
		 * @return array $comments
		 */
		public function FbComments( $comments='' )
		{
			# Comments Closed? Return
			if ( !comments_open() )
				return $comments;
			
			global $wp_query;
			
			$postId = $wp_query->post->ID;
			$postUrl = get_permalink($postId);

			# Only insert Facebook Comments if App ID and Secret is set
			if (!empty($this->fb['app']) && !empty($this->fb['secret']) )
			{
				echo "<div id=\"fbSEOComments\" style=\"width:{$this->fb['width']}\">";
				self::fbAddComment($postUrl, $postId, $comments);
				echo "</div>";
			}
			else // If no application ID or application secret are set ask the user to set one
				self::fbHandleNoAppId();
	
			return $comments;
		}
		
		/**
		 * 
		 * This method will be executed if App ID or Secret can't be found in the options table or if there's any problem with they
		 * @return void
		 */
		public function fbHandleNoAppId()
		{
			get_currentuserinfo(); // Get user info to see if the currently logged in user (if any) has the 'manage_options' capability
	
			if ( !current_user_can('manage_options') )
				echo 'You are not authorized to see this part';
				
			$url = 'options-general.php?page='.$this->name;
			$fb_optionsPage = admin_url($url);
			echo "<div id=\"fbSEOComments\" style=\"width:{$this->fb['width']}\"> Please, insert a valid <a href='$fb_optionsPage' style='color: #c00;'>App ID</a>, otherwise your plugin won't work correctly.</div>";
		}
		
		/**
		 * 
		 * This method is responsible for the logic in adding the facebook comments into the wordpress comments database and also embending the facebook comment form
		 * @param string(required) $postUrl
		 * @param string(required) $postId
		 * @return void or string on error returns message else prints the comments on the page
		 * @TODO improve performance by doing some pre-query so all the checks will be performed only when there is new comments
		 */
		public function fbAddComment($postUrl = '', $postId = '')
		{
			global $wpdb;
			
			if( empty($postUrl) || empty($postId) )
				echo 'You must insert an valid Url and Post ID';

			$fbOutputComments = '';
			$facebook = $this->getFbCommentsApi();
						
			# Lets fetch all the necessary data from Facebook
			# @TODO Maybe create a cache of the comments to improve performance, thousands of loads on this can do some mess I think
			$queries = array('q1' => 'SELECT post_fbid, fromid, object_id, text, time '.
									 'FROM comment '.
									 'WHERE object_id in (SELECT comments_fbid FROM link_stat WHERE url ="'.$postUrl.'")', # '.$postUrl.'
							 'q2' => 'SELECT post_fbid, fromid, object_id, text, time FROM comment WHERE object_id in (SELECT post_fbid FROM #q1)',
							 'q3' => 'SELECT name, id, url, pic_square FROM profile WHERE id in (SELECT fromid FROM #q1) or id in (SELECT fromid FROM #q2)',
							 );
		
			$queries = json_encode($queries);
			$fbquery = array( "method" => "fql.multiquery", "queries" => $queries );
			$result = $facebook->api($fbquery);
            
			# Fetch the Authors of the comments
			$comments['authors'] = $result[2]['fql_result_set'];
			
			# Fetch the comments
			$comments['comments'] = $result[0]['fql_result_set'];
			
			# Fetch the nested comments
			$comments['comments'] = array_merge($comments['comments'], $result[1]['fql_result_set']);
		
			foreach($comments['comments'] as $comment)
			{
				$commentId = $comment['post_fbid'];
				$objId = $comment['object_id'];
				$timestamp = $comment['time'];
				$userCommentId = $comment['fromid'];
				$userName = '';	# name of the commenter
				$userUrl = '';	# url of the commenter
                
				# If the comment dont exists lets add it
				if( !$this->_checkIfCommentExists($commentId) )
				{
					# Check auto approve
					if( empty($this->fb['autoApprove']) || $this->fb['autoApprove'] == null)
						$this->fb['autoApprove'] = 0;

					# Who is the User that commented?
					foreach($comments['authors'] as $user)
					{
						if ($userCommentId == $user['id'])
						{
							$userName = $user['name'];
							$userUrl = $user['url'];
						}
					}
					
					# This comment have any parent?
					$commentParentSql =	   "SELECT comment_ID
											FROM {$this->table}
											WHERE facebook_comment_ID = {$objId}";
									
					$commentParent = $wpdb->get_results($commentParentSql); 
					
					if ($commentParent[0]->comment_ID);
						$parent = $commentParent[0]->comment_ID;
					

                    # Lets strip html data before input in the database
                    $cleanComment = esc_html( $comment['text'] );

                    # Prepare the data to be inserted in the comment database
					$commentdata = array(
						'comment_post_ID' => $postId,
						'comment_author' => $userName,
						'comment_author_email' => '',
						'comment_author_url' => $userUrl,
						'comment_author_IP' => '',
						'comment_date' => date('Y-m-d G:i:s', $timestamp),
						'comment_date_gmt' => date('Y-m-d G:i:s', $timestamp),
						'comment_content' => $cleanComment,
						'comment_karma' => '',
						'comment_approved' => $this->fb['autoApprove'],
						'comment_agent' => '',
						'comment_type' => '',
						'comment_parent' => $parent,
						'user_id' => 0
					);

                    # Insert the comment in the database and also updates the post comment count
                    # ref file wp-includes/comment.php
				    $newCommentId = wp_insert_comment( $commentdata );

                    # Lets keep track of all already inserted comments
					$facebookCommentData = array(
						'comment_ID' => $newCommentId,
						'facebook_comment_ID' => $commentId,
						'facebook_user_ID' => $userCommentId
					);
					
					$wpdb->insert($this->table, $facebookCommentData);
				}
			}

			# Finally Lets output the code that calls the facebook comment plugin with the user configuration
			$fbOutputComments .= "\t<fb:comments
								href='{$postUrl}'
								num_posts='{$this->fb['numPosts']}'
								width='{$this->fb['width']}'
								colorscheme='{$this->fb['colorScheme']}'>
							</fb:comments>";
		
			echo $fbOutputComments ;
		}

        /**
         * This method checks for the existence of the comment in the SEO Facebook Comment table
         *
         * @param string $id
         * @return int
         */
        protected function _checkIfCommentExists( $id = '' )
        {
            if ( empty( $id ) )
                return 0;

            global $wpdb;

            # Check if the comment was created in a load before
            $commentExistsSql = "SELECT comment_ID
                                 FROM {$this->table}
                                 WHERE facebook_comment_ID = {$id}";

            return $wpdb->get_results($commentExistsSql);
        }
		
	}

    $SEOFacebookComments = new SEOFacebookComments;
}